@@ -18,7 +18,7 @@ import multiprocessing as mp
 from scipy import linalg
 from scipy.optimize import OptimizeResult
 from numpy.random import MT19937, Generator
-from fcmaes.evaluator import Evaluator
+from fcmaes.evaluator import Evaluator, serial, _check_bounds, _fitness
 
 os.environ['MKL_DEBUG_CPU_TYPE'] = '5'
 
@@ -119,7 +119,7 @@ class Cmaes(object):
                         max_evaluations = 100000, 
                         max_iterations = 100000,  
                         accuracy = 1.0, 
-                        stop_fitness = np.nan, 
+                        stop_fitness = -math.inf, 
                         is_terminate = None, 
                         rg = Generator(MT19937()), # used if x0 is undefined
                         randn = np.random.randn, # used for random offspring 
@@ -351,8 +351,6 @@ class Cmaes(object):
         return self.stop                
            
     def newArgs(self):
-        self.xmean = self.fitfun.closestFeasible(self.xmean)
-        self.fitness = np.full(self.popsize, math.inf)
         # generate random offspring
         self.arz = self.randn(self.popsize, self.dim)    
         delta = (self.BD @ self.arz.transpose()) * self.sigma
@@ -395,10 +393,9 @@ class Cmaes(object):
             self.iterations += 1
             if self.fitfun.evaluation_counter > self.max_evaluations:
                 break
-            # Generate and evaluate popsize offspring
-            self.newArgs()            
-            self.fitness = self.fitfun.values(self.arx)
-            self.updateCMA()
+            xs = self.ask()
+            ys = self.fitfun.values(xs)
+            self.tell(ys, xs)
             if self.stop != 0:
                 break
         return self.best_x, self.best_value, self.fitfun.evaluation_counter, self.iterations, self.stop 
@@ -414,10 +411,9 @@ class Cmaes(object):
         if self.best_value > best_fitness:
             self.best_value = best_fitness
             self.best_x = self.fitfun.decode(self.arx[arindex[0]])    
-            if self.stop_fitness != None: # only if stop_fitness is defined
-                if best_fitness < self.stop_fitness:
-                    self.stop = 1
-                    return 
+            if best_fitness < self.stop_fitness:
+                self.stop = 1
+                return 
 
         # Calculate new xmean, this is selection and recombination
         xold = self.xmean # for speed up of Eq. (2) and (3)
@@ -589,80 +585,4 @@ class Cmaes(object):
             self.diagD = np.sqrt(self.diagD) # diagD contains standard deviations now
             
             self.BD = self.B * self.diagD # O(n^2)
-
-def serial(fun):
-    """Convert an objective function for serial execution for cmaes.minimize.
-    
-    Parameters
-    ----------
-    fun : objective function mapping a list of float arguments to a float value
-
-    Returns
-    -------
-    out : function
-        A function mapping a list of lists of float arguments to a list of float values
-        by applying the input function in a loop."""
-  
-    return lambda xs : [_tryfun(fun, x) for x in xs]
-        
-def _func_serial(fun, num, pid, xs, ys):
-    for i in range(pid, len(xs), num):
-        ys[i] = _tryfun(fun, xs[i])
-
-def _tryfun(fun, x):
-    try:
-        fit = fun(x)
-        return fit if math.isfinite(fit) else sys.float_info.max
-    except Exception:
-        return sys.float_info.max
-                        
-def _check_bounds(bounds, guess, rg):
-    if bounds is None and guess is None:
-        raise ValueError('either guess or bounds need to be defined')
-    if bounds is None:
-        return None, None, np.asarray(guess)
-    if guess is None:
-        guess = rg.uniform(bounds.lb, bounds.ub)
-    return np.asarray(bounds.lb), np.asarray(bounds.ub), np.asarray(guess)
-
-class _fitness(object):
-    """wrapper around the objective function, scales relative to boundaries."""
-     
-    def __init__(self, fun, lower, upper, normalize = None):
-        self.fun = fun
-        self.evaluation_counter = 0
-        self.lower = lower
-        self.normalize = False
-        if not (lower is None or normalize is None):
-            self.normalize = normalize
-        if not lower is None:
-            self.upper = upper
-            self.scale = 0.5 * (upper - lower)
-            self.typx = 0.5 * (upper + lower)
-                
-    def values(self, Xs): #enables parallel evaluation
-        values = self.fun([self.decode(X) for X in Xs])
-        self.evaluation_counter += len(Xs)
-        return np.array(values)
-
-    def closestFeasible(self, X):
-        if self.lower is None:
-            return X    
-        else:
-            if self.normalize:
-                return np.maximum(np.minimum(X, 1.0), -1.0)
-            else:
-                return np.maximum(np.minimum(X, self.upper), self.lower)
-
-    def encode(self, X):
-        if self.normalize:
-            return (X - self.typx) / self.scale
-        else:
-            return X
-   
-    def decode(self, X):
-        if self.normalize:
-            return (X * self.scale) + self.typx
-        else:
-            return X
-         
+                        
\ No newline at end of file
