@@ -15,8 +15,7 @@ import ctypes as ct
 import numpy as np
 from numpy.random import MT19937, Generator
 from scipy.optimize import OptimizeResult
-from fcmaes.cmaes import _check_bounds
-from fcmaes.decpp import mo_call_back_type, callback, libcmalib
+from fcmaes.evaluator import _check_bounds, mo_call_back_type, callback_so, libcmalib
 
 os.environ['MKL_DEBUG_CPU_TYPE'] = '5'
 
@@ -26,7 +25,7 @@ def minimize(fun,
              input_sigma = 0.166, 
              popsize = 0, 
              max_evaluations = 100000, 
-             stop_fitness = None, 
+             stop_fitness = -math.inf, 
              rg = Generator(MT19937()),
              runid=0):
        
@@ -79,11 +78,9 @@ def minimize(fun,
     if callable(input_sigma):
         input_sigma=input_sigma()
     if np.ndim(input_sigma) == 0:
-        input_sigma = [input_sigma] * dim
-    if stop_fitness is None:
-        stop_fitness = -math.inf   
+        input_sigma = [input_sigma] * dim  
     array_type = ct.c_double * dim 
-    c_callback = mo_call_back_type(callback(fun, dim))
+    c_callback = mo_call_back_type(callback_so(fun, dim))
     res = np.empty(dim+4)
     res_p = res.ctypes.data_as(ct.POINTER(ct.c_double))
     try:
