@@ -6,7 +6,7 @@
 # parallel optimization retry of a multi-objective problem.
 
 import numpy as np
-import math, sys, time, warnings
+import math, sys, time, warnings, threadpoolctl
 import multiprocessing as mp
 from multiprocessing import Process
 from scipy.optimize import Bounds
@@ -111,26 +111,27 @@ def _retry_loop(pid, rgs, fun, weight_bounds, ncon, y_exp,
     lower = store.lower
     wlb = np.array(weight_bounds.lb)
     wub = np.array(weight_bounds.ub)
-    while store.get_runs_compare_incr(num_retries):      
-        try:       
-            rg = rgs[pid]
-            w = rg.uniform(size=len(wub))          
-            w /= _avg_exp(w, y_exp) # correct scaling
-            w = wlb + w * (wub - wlb)
-            wrapper = mo_wrapper(fun, w, ncon, y_exp)  
-            x, y, evals = optimize(wrapper.eval, Bounds(store.lower, store.upper), None, 
-                                     [rg.uniform(0.05, 0.1)]*len(lower), rg, store)
-            objs = wrapper.mo_eval(x) # retrieve the objective values
-            if value_limits is None or all([objs[i] < value_limits[i] for i in range(len(w))]):
-                store.add_result(y, x, evals, math.inf)   
-                if not store.plot_name is None:
-                    name = store.plot_name + "_moretry_" + str(store.get_count_evals())
-                    xs = np.array(store.get_xs())
-                    ys = np.array([fun(x) for x in xs])
-                    np.savez_compressed(name, xs=xs, ys=ys) 
-                    plot(name, ncon, xs, ys)
-        except Exception as ex:
-            print(str(ex))
+    with threadpoolctl.threadpool_limits(limits=1, user_api="blas"):    
+        while store.get_runs_compare_incr(num_retries):      
+            try:       
+                rg = rgs[pid]
+                w = rg.uniform(size=len(wub))          
+                w /= _avg_exp(w, y_exp) # correct scaling
+                w = wlb + w * (wub - wlb)
+                wrapper = mo_wrapper(fun, w, ncon, y_exp)  
+                x, y, evals = optimize(wrapper.eval, Bounds(store.lower, store.upper), None, 
+                                         [rg.uniform(0.05, 0.1)]*len(lower), rg, store)
+                objs = wrapper.mo_eval(x) # retrieve the objective values
+                if value_limits is None or all([objs[i] < value_limits[i] for i in range(len(w))]):
+                    store.add_result(y, x, evals, math.inf)   
+                    if not store.plot_name is None:
+                        name = store.plot_name + "_moretry_" + str(store.get_count_evals())
+                        xs = np.array(store.get_xs())
+                        ys = np.array([fun(x) for x in xs])
+                        np.savez_compressed(name, xs=xs, ys=ys) 
+                        plot(name, ncon, xs, ys)
+            except Exception as ex:
+                print(str(ex))
             
 def pareto(xs, ys):
     """pareto front for argument vectors and corresponding function value vectors."""
@@ -195,9 +196,9 @@ def plot(name, ncon, xs, ys, eps = 1E-2, all=True, interp=False, plot3d=False):
                 return
         if all:
             retry.plot(ys, 'all_' + name + '.png', interp=False)
-            xs, ys = pareto(xs, ys)
-            for x, y in zip(xs, ys):
-                print(str(list(y)) + ' ' + str([int(xi) for xi in x]))
+        xs, ys = pareto(xs, ys)
+        for x, y in zip(xs, ys):
+            print(str(list(y)) + ' ' + str([round(xi,5) for xi in x]))
         retry.plot(ys, 'front_' + name + '.png', interp=interp, plot3d=plot3d)
     except Exception as ex:
         print(str(ex))
