@@ -12,7 +12,7 @@ import math
 import logging
 import ctypes as ct
 import multiprocessing as mp 
-
+from fcmaes.evaluator import serial, parallel
 from fcmaes import crfmnes, crfmnescpp, cmaes, de, cmaescpp, decpp, dacpp, gcldecpp, lcldecpp, ldecpp, csmacpp, bitecpp
 
 _logger = None
@@ -148,70 +148,72 @@ class Choice(Optimizer):
         return opt.minimize(fun, bounds, guess, sdevs, rg, store)
 
 def de_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
-           de_max_evals = None, cma_max_evals = None, ints = None):
+           de_max_evals = None, cma_max_evals = None, ints = None, workers = None):
     """Sequence differential evolution -> CMA-ES."""
 
-    deEvals = np.random.uniform(0.1, 0.5)
+    de_evals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(deEvals*max_evaluations)
+        de_max_evals = int(de_evals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int((1.0-deEvals)*max_evaluations)
+        cma_max_evals = int((1.0-de_evals)*max_evaluations)
     opt1 = De_cpp(popsize=popsize, max_evaluations = de_max_evals, 
-                  stop_fitness = stop_fitness, ints=ints)
+                  stop_fitness = stop_fitness, ints=ints, workers = workers)
     opt2 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
-                   stop_fitness = stop_fitness)
+                   stop_fitness = stop_fitness, workers = workers)
     return Sequence([opt1, opt2])
 
 def de_cma_py(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
-           de_max_evals = None, cma_max_evals = None, ints = None):
+           de_max_evals = None, cma_max_evals = None, ints = None, workers = None):
     """Sequence differential evolution -> CMA-ES in python."""
 
-    deEvals = np.random.uniform(0.1, 0.5)
+    de_evals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(deEvals*max_evaluations)
+        de_max_evals = int(de_evals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int((1.0-deEvals)*max_evaluations)
+        cma_max_evals = int((1.0-de_evals)*max_evaluations)
     opt1 = De_python(popsize=popsize, max_evaluations = de_max_evals, 
-                     stop_fitness = stop_fitness, ints=ints)
+                     stop_fitness = stop_fitness, ints=ints, workers = workers)
     opt2 = Cma_python(popsize=popsize, max_evaluations = cma_max_evals, 
-                   stop_fitness = stop_fitness)
+                   stop_fitness = stop_fitness, workers = workers)
     return Sequence([opt1, opt2])
 
 def de2_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
-           de_max_evals = None, cma_max_evals = None, ints = None):
+           de_max_evals = None, cma_max_evals = None, ints = None, workers = None):
     """Sequence differential evolution -> CMA-ES."""
 
-    deEvals = np.random.uniform(0.1, 0.5)
+    de_evals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(deEvals*max_evaluations)
+        de_max_evals = int(de_evals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int((1.0-deEvals)*max_evaluations)
-    opt1 = Choice([GCLDE_cpp(de_max_evals), De_cpp(de_max_evals, ints=ints)])
-    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fitness = stop_fitness)
+        cma_max_evals = int((1.0-de_evals)*max_evaluations)
+    opt1 = Choice([GCLDE_cpp(de_max_evals, workers = workers), 
+                   De_cpp(de_max_evals, ints=ints, workers = workers)])
+    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fitness = stop_fitness, workers = workers)
     return Sequence([opt1, opt2])
 
 def de3_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
-           de_max_evals = None, cma_max_evals = None):
+           de_max_evals = None, cma_max_evals = None, workers = None):
     """Sequence differential evolution -> CMA-ES."""
 
-    deEvals = np.random.uniform(0.1, 0.5)
+    de_evals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(deEvals*max_evaluations)
+        de_max_evals = int(de_evals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int((1.0-deEvals)*max_evaluations)
-    opt1 =  Choice([GCLDE_cpp(de_max_evals), Cma_cpp(de_max_evals), De_cpp(de_max_evals)])
-    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fitness = stop_fitness)
+        cma_max_evals = int((1.0-de_evals)*max_evaluations)
+    opt1 =  Choice([GCLDE_cpp(de_max_evals), Cma_cpp(de_max_evals, workers = workers), 
+                    De_cpp(de_max_evals, workers = workers)])
+    opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fitness = stop_fitness, workers = workers)
     return Sequence([opt1, opt2])
 
 def gclde_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
            de_max_evals = None, cma_max_evals = None, workers = None):
     """Sequence G-CL-differential evolution -> CMA-ES."""
 
-    deEvals = np.random.uniform(0.1, 0.5)
+    de_evals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(deEvals*max_evaluations)
+        de_max_evals = int(de_evals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int((1.0-deEvals)*max_evaluations)
+        cma_max_evals = int((1.0-de_evals)*max_evaluations)
     opt1 = GCLDE_cpp(max_evaluations = de_max_evals, stop_fitness = stop_fitness, workers = workers)
     opt2 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
                    stop_fitness = stop_fitness, workers = workers)
@@ -219,13 +221,13 @@ def gclde_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf,
 
 def da_cma(max_evaluations = 50000, da_max_evals = None, cma_max_evals = None,
            popsize=31, stop_fitness = -math.inf):
-    """Sequence differential evolution -> CMA-ES."""
+    """Sequence dual annealing -> CMA-ES."""
 
-    daEvals = np.random.uniform(0.1, 0.5)
+    da_evals = np.random.uniform(0.1, 0.5)
     if da_max_evals is None:
-        da_max_evals = int(daEvals*max_evaluations)
+        da_max_evals = int(da_evals*max_evaluations)
     if cma_max_evals is None:
-        cma_max_evals = int((1.0-daEvals)*max_evaluations)
+        cma_max_evals = int((1.0-da_evals)*max_evaluations)
     opt1 = Da_cpp(max_evaluations = da_max_evals, stop_fitness = stop_fitness)
     opt2 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
                    stop_fitness = stop_fitness)
@@ -233,13 +235,13 @@ def da_cma(max_evaluations = 50000, da_max_evals = None, cma_max_evals = None,
 
 def de_crfmnes(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
            de_max_evals = None, crfm_max_evals = None, ints = None):
-    """Sequence differential evolution -> CMA-ES."""
+    """Sequence differential evolution -> CRFMNES."""
 
-    deEvals = np.random.uniform(0.1, 0.5)
+    de_evals = np.random.uniform(0.1, 0.5)
     if de_max_evals is None:
-        de_max_evals = int(deEvals*max_evaluations)
+        de_max_evals = int(de_evals*max_evaluations)
     if crfm_max_evals is None:
-        crfm_max_evals = int((1.0-deEvals)*max_evaluations)
+        crfm_max_evals = int((1.0-de_evals)*max_evaluations)
     opt1 = De_cpp(popsize=popsize, max_evaluations = de_max_evals, 
                   stop_fitness = stop_fitness, ints=ints)
     opt2 = Crfmnes_cpp(popsize=popsize, max_evaluations = crfm_max_evals, 
@@ -248,30 +250,46 @@ def de_crfmnes(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf,
 
 def crfmnes_bite(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
            crfm_max_evals = None, bite_max_evals = None, M=1):
-    """Sequence differential evolution -> CMA-ES."""
+    """Sequence CRFMNES -> Bite."""
 
-    deEvals = np.random.uniform(0.1, 0.5)
+    crfmnes_evals = np.random.uniform(0.1, 0.5)
     if crfm_max_evals is None:
-        crfm_max_evals = int(deEvals*max_evaluations)
+        crfm_max_evals = int(crfmnes_evals*max_evaluations)
     if bite_max_evals is None:
-        bite_max_evals = int((1.0-deEvals)*max_evaluations)
+        bite_max_evals = int((1.0-crfmnes_evals)*max_evaluations)
     opt1 = Crfmnes_cpp(popsize=popsize, max_evaluations = crfm_max_evals, 
                   stop_fitness = stop_fitness)
     opt2 = Bite_cpp(popsize=popsize, max_evaluations = bite_max_evals, 
                    stop_fitness = stop_fitness, M=M)
     return Sequence([opt1, opt2])
 
+def cma_bite(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
+           cma_max_evals = None, bite_max_evals = None, M=1):
+    """Sequence CMA-ES -> Bite."""
+
+    cma_evals = np.random.uniform(0.1, 0.5)
+    if cma_max_evals is None:
+        cma_max_evals = int(cma_evals*max_evaluations)
+    if bite_max_evals is None:
+        bite_max_evals = int((1.0-cma_evals)*max_evaluations)
+    opt1 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
+                  stop_fitness = stop_fitness, stop_hist = 0)
+    opt2 = Bite_cpp(popsize=popsize, max_evaluations = bite_max_evals, 
+                   stop_fitness = stop_fitness, M=M)
+    return Sequence([opt1, opt2])
+
 class Crfmnes(Optimizer):
     """CRFMNES Python implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = 32, guess=None, stop_fitness = None,
-                 sdevs = None):        
+                 popsize = 32, guess=None, stop_fitness = -math.inf,
+                 sdevs = None, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'crfmnes')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
         self.guess = guess
         self.sdevs = sdevs
+        self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         ret = crfmnes.minimize(fun, bounds, 
@@ -280,20 +298,22 @@ class Crfmnes(Optimizer):
                 max_evaluations = self.max_eval_num(store), 
                 popsize=self.popsize, 
                 stop_fitness = self.stop_fitness,
-                rg=rg, runid=self.get_count_runs(store))     
+                rg=rg, runid=self.get_count_runs(store),
+                workers = self.workers)     
         return ret.x, ret.fun, ret.nfev
 
 class Crfmnes_cpp(Optimizer):
     """CRFMNES C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = 32, guess=None, stop_fitness = None,
-                 sdevs = None):        
+                 popsize = 32, guess=None, stop_fitness = -math.inf,
+                 sdevs = None, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'crfmnes cpp')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
         self.guess = guess
         self.sdevs = sdevs
+        self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         ret = crfmnescpp.minimize(fun, bounds, 
@@ -302,21 +322,24 @@ class Crfmnes_cpp(Optimizer):
                 max_evaluations = self.max_eval_num(store), 
                 popsize=self.popsize, 
                 stop_fitness = self.stop_fitness,
-                rg=rg, runid=self.get_count_runs(store))     
+                rg=rg, runid=self.get_count_runs(store), 
+                workers = self.workers)     
         return ret.x, ret.fun, ret.nfev
 
 class Cma_python(Optimizer):
     """CMA_ES Python implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fitness = None,
-                 update_gap = None, sdevs = None):        
+                 popsize = 31, guess=None, stop_fitness = -math.inf,
+                 update_gap = None, sdevs = None, normalize = True, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'cma py')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
         self.update_gap = update_gap
         self.guess = guess
         self.sdevs = sdevs
+        self.normalize = normalize
+        self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         ret = cmaes.minimize(fun, bounds, 
@@ -326,15 +349,17 @@ class Cma_python(Optimizer):
                 popsize=self.popsize, 
                 stop_fitness = self.stop_fitness,
                 rg=rg, runid=self.get_count_runs(store),
-                update_gap = self.update_gap)     
+                normalize = self.normalize,
+                update_gap = self.update_gap,
+                workers = self.workers)     
         return ret.x, ret.fun, ret.nfev
 
 class Cma_cpp(Optimizer):
     """CMA_ES C++ implementation."""
    
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fitness = None, stop_hist = None, 
-                 update_gap = None, workers = None, sdevs = None):        
+                 popsize = 31, guess=None, stop_fitness = -math.inf, stop_hist = None, 
+                 update_gap = None, delayed_update = True, sdevs = None, normalize = True, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'cma cpp')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
@@ -342,27 +367,30 @@ class Cma_cpp(Optimizer):
         self.guess = guess
         self.sdevs = sdevs
         self.update_gap = update_gap
+        self.delayed_update = delayed_update
+        self.normalize = normalize
         self.workers = workers
 
-    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()),
-                 store=None, workers=None):
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         ret = cmaescpp.minimize(fun, bounds,
                 self.guess if not self.guess is None else guess,
-                input_sigma= self.sdevs if not self.sdevs is None else sdevs,
-                max_evaluations=self.max_eval_num(store),
-                popsize=self.popsize,
-                stop_fitness=self.stop_fitness,
-                stop_hist=self.stop_hist,
-                rg=rg, runid=self.get_count_runs(store),
-		              update_gap=self.update_gap,
-                workers=self.workers if workers is None else workers)     
+                input_sigma = self.sdevs if not self.sdevs is None else sdevs,
+                max_evaluations =self.max_eval_num(store),
+                popsize = self.popsize,
+                stop_fitness = self.stop_fitness,
+                stop_hist = self.stop_hist,
+                rg = rg, runid = self.get_count_runs(store),
+		        update_gap = self.update_gap,
+                normalize = self.normalize,
+                delayed_update = self.delayed_update,
+                workers = self.workers)   
         return ret.x, ret.fun, ret.nfev
 
 class Cma_orig(Optimizer):
     """CMA_ES original implementation."""
    
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fitness = None, sdevs = None):        
+                 popsize = None, guess=None, stop_fitness = -math.inf, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'cma orig')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
@@ -401,13 +429,155 @@ class Cma_orig(Optimizer):
             return es.result.xbest, es.result.fbest, evals
         except Exception as ex:
             print(ex)
-  
+
+class Cma_lw(Optimizer):
+    """CMA lightweight Python implementation. See https://github.com/CyberAgentAILab/cmaes """
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = None, guess=None, stop_fitness = None,
+                 sdevs = None, workers = None):        
+        Optimizer.__init__(self, max_evaluations, 'cma_lw')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
+        self.guess = guess
+        self.sdevs = sdevs
+        self.workers = workers
+
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
+        try:
+            import cmaes
+        except ImportError as e:
+            raise ImportError("Please install cmaes (pip install cmaes)") 
+
+        if guess is None:
+            guess = self.guess
+        if guess is None:    
+            guess = rg.uniform(bounds.lb, bounds.ub)
+        bds = np.array([t for t in zip(bounds.lb, bounds.ub)])
+        seed = int(rg.uniform(0, 2**32 - 1))
+        optimizer = cmaes.CMA(mean=guess, sigma=np.mean(sdevs), bounds=bds, seed=seed, population_size=self.popsize)
+        best_y = math.inf
+        evals = 0
+        fun = serial(fun) if (self.workers is None or self.workers <= 1) else parallel(fun, self.workers)  
+        while evals < self.max_evaluations and not optimizer.should_stop():
+            xs = [optimizer.ask() for _ in range(optimizer.population_size)]
+            ys = fun(xs)
+            solutions = []
+            for i in range(optimizer.population_size):
+                x = xs[i]
+                y = ys[i]
+                solutions.append((x, y))
+                if y < best_y:
+                    best_y = y
+                    best_x = x
+            optimizer.tell(solutions)
+            evals += optimizer.population_size           
+        if isinstance(fun, parallel):
+            fun.stop()
+        return best_x, best_y, evals
+
+class Cma_awm(Optimizer):
+    """CMA awm Python implementation. See https://github.com/CyberAgentAILab/cmaes """
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = None, guess=None, stop_fitness = None,
+                 sdevs = None, continuous_space=None, discrete_space=None, workers = None):        
+        Optimizer.__init__(self, max_evaluations, 'cma_awm')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
+        self.guess = guess
+        self.sdevs = sdevs
+        self.workers = workers
+        self.continuous_space = continuous_space
+        self.discrete_space = discrete_space
+
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
+        try:
+            import cmaes
+        except ImportError as e:
+            raise ImportError("Please install cmaes (pip install cmaes)") 
+              
+        if guess is None:
+            guess = self.guess
+        if guess is None:    
+            guess = rg.uniform(bounds.lb, bounds.ub)
+        seed = int(rg.uniform(0, 2**32 - 1))
+        optimizer = cmaes.CMAwM(mean=guess, sigma=np.mean(sdevs),       
+                         continuous_space=self.continuous_space,  
+                         discrete_space=self.discrete_space, 
+                         seed=seed, population_size=self.popsize)
+        best_y = 1E99
+        evals = 0
+        fun = serial(fun) if (self.workers is None or self.workers <= 1) else parallel(fun, self.workers)  
+        while evals < self.max_evaluations and not optimizer.should_stop():
+            asks = [optimizer.ask() for _ in range(optimizer.population_size)]
+            ys = fun([x[0] for x in asks])
+            solutions = []
+            for i in range(optimizer.population_size):
+                x = asks[i][1]
+                y = ys[i]
+                solutions.append((x, y))
+                if y < best_y:
+                    best_y = y
+                    best_x = x
+            optimizer.tell(solutions)
+            evals += optimizer.population_size           
+        if isinstance(fun, parallel):
+            fun.stop()
+        return best_x, best_y, evals
+
+class Cma_sep(Optimizer):
+    """CMA sep Python implementation. See https://github.com/CyberAgentAILab/cmaes """
+    
+    def __init__(self, max_evaluations=50000,
+                 popsize = 32, guess=None, stop_fitness = None,
+                 sdevs = None, workers = None):        
+        Optimizer.__init__(self, max_evaluations, 'cma_sep')
+        self.popsize = popsize
+        self.stop_fitness = stop_fitness
+        self.guess = guess
+        self.sdevs = sdevs
+        self.workers = workers
+
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
+        try:
+            import cmaes
+        except ImportError as e:
+            raise ImportError("Please install cmaes (pip install cmaes)") 
+
+        if guess is None:
+            guess = self.guess
+        if guess is None:    
+            guess = rg.uniform(bounds.lb, bounds.ub)
+        bds = np.array([t for t in zip(bounds.lb, bounds.ub)])
+        seed = int(rg.uniform(0, 2**32 - 1))
+        optimizer = cmaes.SepCMA(mean=guess, sigma=np.mean(sdevs), bounds=bds, seed=seed, population_size=self.popsize)
+        best_y = math.inf
+        evals = 0
+        fun = serial(fun) if (self.workers is None or self.workers <= 1) else parallel(fun, self.workers)  
+        while evals < self.max_evaluations and not optimizer.should_stop():
+            xs = [optimizer.ask() for _ in range(optimizer.population_size)]
+            ys = fun(xs)
+            solutions = []
+            for i in range(optimizer.population_size):
+                x = xs[i]
+                y = ys[i]
+                solutions.append((x, y))
+                if y < best_y:
+                    best_y = y
+                    best_x = x
+            optimizer.tell(solutions)
+            evals += optimizer.population_size          
+        if isinstance(fun, parallel):
+            fun.stop()
+        return best_x, best_y, evals
+      
 class De_cpp(Optimizer):
     """Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fitness = None, 
-                 keep = 200, f = 0.5, cr = 0.9, ints = None):        
+                 popsize = None, stop_fitness = -math.inf, 
+                 keep = 200, f = 0.5, cr = 0.9, ints = None, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'de cpp')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
@@ -415,6 +585,7 @@ class De_cpp(Optimizer):
         self.f = f
         self.cr = cr
         self.ints = ints
+        self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
         ret = decpp.minimize(fun, None, bounds, 
@@ -422,15 +593,16 @@ class De_cpp(Optimizer):
                 max_evaluations = self.max_eval_num(store), 
                 stop_fitness = self.stop_fitness,
                 keep = self.keep, f = self.f, cr = self.cr, ints=self.ints,
-                rg=rg, runid = self.get_count_runs(store))
+                rg=rg, runid = self.get_count_runs(store), 
+                workers = self.workers)
         return ret.x, ret.fun, ret.nfev
 
 class De_python(Optimizer):
     """Differential Evolution Python implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fitness = None, 
-                 keep = 200, f = 0.5, cr = 0.9, ints = None):        
+                 popsize = None, stop_fitness = -math.inf, 
+                 keep = 200, f = 0.5, cr = 0.9, ints = None, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'de py')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
@@ -438,20 +610,21 @@ class De_python(Optimizer):
         self.f = f
         self.cr = cr
         self.ints = ints
+        self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
         ret = de.minimize(fun, None, 
-                bounds, self.popsize, self.max_eval_num(store), workers=None,
+                bounds, self.popsize, self.max_eval_num(store),
                 stop_fitness = self.stop_fitness,
                 keep = self.keep, f = self.f, cr = self.cr, ints=self.ints,
-                rg=rg)
+                rg=rg, workers = self.workers)
         return ret.x, ret.fun, ret.nfev
 
 class Cma_ask_tell(Optimizer):
     """CMA ask tell implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fitness = None, sdevs = None):        
+                 popsize = 31, guess=None, stop_fitness = -math.inf, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'cma at')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
@@ -478,7 +651,7 @@ class De_ask_tell(Optimizer):
     """Differential Evolution ask tell implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fitness = None, 
+                 popsize = None, stop_fitness = -math.inf, 
                  keep = 400, f = 0.5, cr = 0.9):        
         Optimizer.__init__(self, max_evaluations, 'de at')
         self.popsize = popsize
@@ -526,7 +699,7 @@ class LDe_cpp(Optimizer):
     """Local Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fitness = None, 
+                 popsize = None, stop_fitness = -math.inf, 
                  keep = 200, f = 0.5, cr = 0.9, guess = None, 
                  sdevs = None, ints = None):        
         Optimizer.__init__(self, max_evaluations, 'lde cpp')
@@ -554,7 +727,7 @@ class GCLDE_cpp(Optimizer):
     """GCL-Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fitness = None, 
+                 popsize = None, stop_fitness = -math.inf, 
                  pbest = 0.7, f0 = 0.0, cr0 = 0.0, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'gclde cpp')
         self.popsize = popsize
@@ -564,23 +737,22 @@ class GCLDE_cpp(Optimizer):
         self.cr0 = cr0
         self.workers = workers
 
-    def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), 
-                 store=None, workers = None):
+    def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), store=None):
         ret = gcldecpp.minimize(fun, None, bounds, 
                 popsize=self.popsize, 
                 max_evaluations = self.max_eval_num(store), 
                 stop_fitness = self.stop_fitness,
                 pbest = self.pbest, f0 = self.f0, cr0 = self.cr0,
                 rg=rg, runid = self.get_count_runs(store),
-                workers = self.workers if workers is None else workers)
+                workers = self.workers)
         return ret.x, ret.fun, ret.nfev
 
 class LCLDE_cpp(Optimizer):
     """LCL-Differential Evolution C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 popsize = None, stop_fitness = None, 
-                 pbest = 0.7, f0 = 0.0, cr0 = 0.0, workers = None, guess = None, sdevs = None):        
+                 popsize = None, stop_fitness = -math.inf, 
+                 pbest = 0.7, f0 = 0.0, cr0 = 0.0, guess = None, sdevs = None, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'lclde cpp')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
@@ -591,8 +763,7 @@ class LCLDE_cpp(Optimizer):
         self.guess = guess
         self.sdevs = sdevs
 
-    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), 
-                 store=None, workers = None):
+    def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         ret = lcldecpp.minimize(fun, bounds, 
                 self.guess if not self.guess is None else guess, 
                 self.sdevs if not self.sdevs is None else sdevs,
@@ -601,7 +772,7 @@ class LCLDE_cpp(Optimizer):
                 stop_fitness = self.stop_fitness,
                 pbest = self.pbest, f0 = self.f0, cr0 = self.cr0,
                 rg=rg, runid = self.get_count_runs(store),
-                workers = self.workers if workers is None else workers)
+                workers = self.workers)
 
         return ret.x, ret.fun, ret.nfev
     
@@ -609,7 +780,7 @@ class Da_cpp(Optimizer):
     """Dual Annealing C++ implementation."""
     
     def __init__(self, max_evaluations=50000,
-                 stop_fitness = None, use_local_search=True, guess = None):        
+                 stop_fitness = -math.inf, use_local_search=True, guess = None):        
         Optimizer.__init__(self, max_evaluations, 'da cpp',)
         self.stop_fitness = stop_fitness
         self.use_local_search = use_local_search
@@ -627,16 +798,15 @@ class Csma_cpp(Optimizer):
     """SCMA C++ implementation."""
    
     def __init__(self, max_evaluations=50000,
-                 popsize = None, guess=None, stop_fitness = None, workers = None, sdevs = None):        
+                 popsize = None, guess=None, stop_fitness = -math.inf, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'scma cpp')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
         self.guess = guess
-        self.workers = workers
         self.sdevs = sdevs
 
     def minimize(self, fun, bounds, guess=None, sdevs=0.16, rg=Generator(MT19937()), 
-                 store=None, workers = None):
+                 store=None):
         ret = csmacpp.minimize(fun, bounds, 
                 self.guess if guess is None else guess,
                 self.sdevs if not self.sdevs is None else sdevs,
@@ -649,18 +819,17 @@ class Bite_cpp(Optimizer):
     """Bite C++ implementation."""
    
     def __init__(self, max_evaluations=50000, 
-                 guess=None, stop_fitness = None, M = None, popsize = None, 
-                 stall_criterion = None, workers = None):        
+                 guess=None, stop_fitness = -math.inf, M = None, popsize = None, 
+                 stall_criterion = None):        
         Optimizer.__init__(self, max_evaluations, 'bite cpp')
         self.guess = guess
         self.stop_fitness = stop_fitness
         self.M = 1 if M is None else M 
         self.popsize = 0 if popsize is None else popsize 
         self.stall_criterion = 0 if stall_criterion is None else stall_criterion 
-        self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), 
-                 store=None, workers = None):
+                 store=None):
         
         ret = bitecpp.minimize(fun, bounds, 
                 self.guess if guess is None else guess,
